import java.awt.Color;
import java.awt.Container;
import java.awt.Font;

import javax.swing.*;
import javax.swing.table.*;

import jpdf_common.gui.component.table.RowNumberTable;


/**
 * Class UsStatesTable implements a table which contains a information about the fifty states
 * that make up the United States of America.
 */
public class UsStatesTable
  extends JTable
{
  //-----------------------------------------------------------------------
  // Class variables/constants.
  //-----------------------------------------------------------------------
  private static final Icon US_FLAG_ICON = new ImageIcon("USFlag.gif");
    
  
  //-----------------------------------------------------------------------
  // Constructors.
  //-----------------------------------------------------------------------

  /**
   * Creates an instance of this class.
   */
  public UsStatesTable()
  {
    super(createTableData(), createColumnNames());   
    
    configureTable();
    configureColumns();
  }  
  
  /**
   * Creates an instance of this class using the given Data Model.
   * NOTE: The presence of this constructor allows the Table Component Printer to create a clone
   *       of an existing Us States Table for printing purposes -- without the need to instantiate 
   *       a new Data Model.
   */
  public UsStatesTable(TableModel theTableModel)
  {
    super(theTableModel); 
    
    configureTable();
    configureColumns();
  }
  
  
  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------
  
  /**
   * Determines if the cell at the given location is editable.  Hardcoded to return false.
   */
  public boolean isCellEditable(int theRowIndex, int theColIndex)
  {
    return false;
  }
  
  
  //-----------------------------------------------------------------------
  // Protected methods.
  //-----------------------------------------------------------------------

  /**
   * Configures the Scroll Pane which contains this table to make use of a Row Number Table to
   * provide row headers for each row in this table.  This method is automatically called by
   * Swing when this table is placed in a Scroll Pane.
   */
  protected void configureEnclosingScrollPane()
  {
    RowNumberTable theRowNumberTable = null;
    Container      theParent = getParent();
    JScrollPane    theEnclosingScrollPane = null;
    JLabel         theIconLabel = null;

    // Do the standard Scroll Pane configuration.
    super.configureEnclosingScrollPane();

    // If the parent container is a Viewport...
    if (theParent instanceof JViewport)
    {
      // If the grandparent container is a Scroll Pane, then...
      theParent = theParent.getParent();
      if (theParent instanceof JScrollPane)
      {
        theEnclosingScrollPane = (JScrollPane)theParent;
        
        // Create a Row Number Table and set it as the Row Header in the Scroll Pane.
        theRowNumberTable = new RowNumberTable(this, true);       
        theEnclosingScrollPane.setRowHeaderView(theRowNumberTable);
        theEnclosingScrollPane.getRowHeader().setPreferredSize(theRowNumberTable.getPreferredSize());
        
        // Set the Upper-Left Corner Component.
        theIconLabel = new JLabel(US_FLAG_ICON);
        theEnclosingScrollPane.setCorner(JScrollPane.UPPER_LEFT_CORNER, theIconLabel);
      }
    }
  }

  
  //-----------------------------------------------------------------------
  // Private methods.
  //-----------------------------------------------------------------------

  /**
   * Configure this table's columns. 
   */
  private void configureColumns()
  {
    final int COL_WIDTH = 110;
    
    // Set the column widths.
    getColumn("Name").setPreferredWidth(COL_WIDTH);
    getColumn("Statehood Date").setPreferredWidth(COL_WIDTH);
    getColumn("Capital").setPreferredWidth(COL_WIDTH);   
  }  
  
  /**
   * Configure this table. 
   */
  private void configureTable()
  {
    JTableHeader theTableHeader = getTableHeader();
    Font         theFont = new Font("Sans Serif", Font.PLAIN, 12);
    
    // Configure the table.
    setFont(theFont);
    setBackground(new Color(244,244,155));
    setAutoResizeMode(AUTO_RESIZE_OFF);
    
    // Configure the table header.
    theTableHeader.setFont(theFont);
    theTableHeader.setForeground(Color.BLUE);       
  }  
  
  /**
   * Creates the table column names.
   */
  private static String[] createColumnNames()
  {
    return new String[] {"Name", "Statehood Date", "Capital"};
  }  
  
  /**
   * Creates the table data.
   */
  private static Object[][] createTableData()
  {
    return new Object[][] 
      {
        {"Alabama","Dec. 14, 1819","Montgomery"},
        {"Alaska","Jan. 3, 1959","Juneau"},
        {"Arizona","Feb. 14, 1912","Phoenix"},
        {"Arkansas","June 15, 1836","Little Rock"},
        {"California","Sept. 9, 1850","Sacramento"},
        {"Colorado","Aug. 1, 1876","Denver"},
        {"Connecticut","Jan. 9, 1788","Hartford"},
        {"Delaware","Dec. 7, 1787","Dover"},
        {"Florida","March 3, 1845","Tallahassee"},
        {"Georgia","Jan. 2, 1788","Atlanta"},
        {"Hawaii","Aug. 21, 1959","Honolulu"},
        {"Idaho","July 3, 1890","Boise"},
        {"Illinois","Dec. 3, 1818","Springfield"},
        {"Indiana","Dec. 11, 1816","Indianapolis"},
        {"Iowa","Dec. 28, 1846","Des Moines"},
        {"Kansas","Jan. 29, 1861","Topeka"},
        {"Kentucky","June 1, 1792","Frankfort"},
        {"Louisiana","April 30, 1812","Baton Rouge"},
        {"Maine","March 15, 1820","Augusta"},
        {"Maryland","April 28, 1788","Annapolis"},
        {"Massachusetts","Feb. 6, 1788","Boston"},
        {"Michigan","Jan. 26, 1837","Lansing"},
        {"Minnesota","May 11, 1858","Saint Paul"},
        {"Mississippi","Dec. 10, 1817","Jackson"},
        {"Missouri","Aug. 10, 1821","Jefferson City"},
        {"Montana","Nov. 8, 1889","Helena"},
        {"Nebraska","March 1, 1867","Lincoln"},
        {"Nevada","Oct. 31, 1864","Carson City"},
        {"New Hampshire","June 21, 1788","Concord"},
        {"New Jersey","Dec. 18, 1787","Trenton"},
        {"New Mexico","Jan. 6, 1912","Santa Fe"},
        {"New York","July 26, 1788","Albany"},
        {"North Carolina","Nov. 21, 1789","Raleigh"},
        {"North Dakota","Nov. 2, 1889","Bismarck"},
        {"Ohio","March 1, 1803","Columbus"},
        {"Oklahoma","Nov. 16, 1907","Oklahoma City"},
        {"Oregon","Feb. 14, 1859","Salem"},
        {"Pennsylvania","Dec. 12, 1787","Harrisburg"},
        {"Rhode Island","May 29, 1790","Providence"},
        {"South Carolina","May 23, 1788","Columbia"},
        {"South Dakota","Nov. 2, 1889","Pierre"},
        {"Tennessee","June 1, 1796","Nashville"},
        {"Texas","Dec. 29, 1845","Austin"},
        {"Utah","Jan. 4, 1896","Salt Lake City"},
        {"Vermont","March 4, 1791","Montpelier"},
        {"Virginia","June 25, 1788","Richmond"},
        {"Washington","Nov. 11, 1889","Olympia"},
        {"West Virginia","June 20, 1863","Charleston"},
        {"Wisconsin","May 29, 1848","Madison"},
        {"Wyoming","July 10, 1890","Cheyenne"}       
      };
  }

}  